#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <grp.h>

#include <error.h>
#include <errno.h>

int
test_chmod ()
{
	const char *filename = "test_chmod";
	struct stat st;
	int     fd;
	int     rc = 0;

	unlink (filename);

	fd = open (filename, O_CREAT | O_EXCL);
	if (fd < 0)
		error (1, errno, "open");

	/*test chmod */
	chmod (filename, 0667);

	if (stat (filename, &st))
		error (1, errno, "stat");
	else
	{
		if ((st.st_mode & (~S_IFMT)) == 0667)
		{
			puts ("chmod: mode changed");
		} else
		{
			puts ("chmod: mode unchanged");
			rc = 1;
		}
	}

	fchmod (fd, 0666);

	if (stat (filename, &st))
		error (1, errno, "stat");
	else
	{
		if ((st.st_mode & (~S_IFMT)) == 0666)
		{
			puts ("fchmod: mode changed");
		} else
		{
			puts ("fchmod: mode unchanged");
			rc = 1;
		}
	}

	close (fd);

	return rc;
}

void
test_chown ()
{
	const char *filename = "test_chown";
	int     fd;
	int     ret = 0;

	unlink (filename);

	fd = open (filename, O_CREAT | O_EXCL);
	if (fd < 0)
		error (1, errno, "open");

	/*test for segfaults only */
	ret = chown (filename, -1, -1);
	if (ret)
		error (1, errno, "chown");

	ret = fchown (fd, -1, -1);
	if (ret)
		error (1, errno, "fchown");

	ret = lchown (filename, -1, -1);
	if (ret)
		error (1, errno, "lchown");

}

int
main ()
{
	int     rc = 0;

	rc = test_chmod ();
	test_chown ();
	return rc;
}
